<?php

/**
 *
 * $Id: class.TaskFinish.php,v 1.7 2005/09/12 20:26:50 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.7 $
 *
 */

/**
 * coWiki - Installer task class
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 */
class TaskFinish extends AbstractTask {

    const STRING_FINISH_AND_LAUNCH = 'Launch your coWiki now ...';

    // --------------------------------------------------------------------

    public function __construct($Tpl) {
        parent::__construct($Tpl);
        parent::checkValidUser();
        parent::loadClassLoader();
    }

    // --------------------------------------------------------------------

    public function isRequired() {
        return true;
    }

    // --------------------------------------------------------------------

    public function validate() {
        if (@$_POST['submit'] == self::STRING_FINISH_AND_LAUNCH) {
            redirectTo('../');
        }

        return true;
    }

    // --------------------------------------------------------------------

    public function render() {

        $sHtDocs = $this->State->getHtdocsPath();
        $sIncludes = $this->State->getIncludesPath();

        // Store the data we gathered to the configuration file
        try {
            $this->storeConfigSettings();
            $this->storeIncludePathSettings();
            $this->sealInstallation();

            $sStr = '<b>Congratulation, the '.COWIKI_FULL_IDENT.'
                     installation setup has been cleanly finished.
                     Everything seems to be updated and seems to work
                     properly.</b>
                     <br/><br/>
    
                     All setting has been saved in the configuration file
                     (<tt>core.conf</tt>) that is located in
                     <tt>'.$sIncludes.'/</tt> and your installation has
                     been sealed. Please <i>do not remove</i> the
                     control <tt>install.seal</tt> file that has been
                     created in the <tt>'.$sHtDocs.'/</tt> directory!
                     <br/><br/>
    
                     Thank you for the interest in this sofware. If you
                     encounter problems or have suggestions, please visit
                     the <a href="'.COWIKI_HOME.'">official coWiki site</a>.
                     
                     <br/><br/>
                     <br/><br/>
    
                     <center>
                        <input type="submit" class="submit" name="submit"
                        value="'.self::STRING_FINISH_AND_LAUNCH.'">
                     </center>';

        } catch (IOException $ioe) {
            $sStr = '<b class="warn">';
            $sStr .=  'Error: '.$ioe->getMessage();
            $sStr .= '</b>';   
        }

        $sStr .=  '<br/><br/>';

        // Define content
        $this->Tpl->assign('CONTENT', $sStr);

        // Define informational/warning/error messages
        $this->Tpl->assign('MESSAGE', $this->getMsgQueueFormatted());

        echo $this->Tpl->parse(SETUP_PATH.'/tpl/window.default.tpl');
    }

    // --------------------------------------------------------------------

    private function storeConfigSettings() {
        $Conf = $this->getConfig();

        // [ABOUT] section
        $Conf->setSectionKeyValue(
            'ABOUT', 'TITLE', $this->State->getAboutTitle()
        );
        $Conf->setSectionKeyValue(
            'ABOUT', 'DESCRIPTION', $this->State->getAboutDescription()
        );

        // ----------------------------------------------------------------

        // [SOFTWARE] section
        $Conf->setSectionKeyValue(
            'SOFTWARE', 'DEBUG_LEVEL', $this->State->getDebugLevelString()
        );

        // ----------------------------------------------------------------

        // [MAIL] section
        $Conf->setSectionKeyValue(
            'MAIL', 'FULL_NAME', $this->State->getMailFullName()
        );
        $Conf->setSectionKeyValue(
            'MAIL', 'RETURN_PATH', $this->State->getMailReturnPath()
        );
        $Conf->setSectionKeyValue(
            'MAIL', 'ABUSE_PATH', $this->State->getMailAbusePath()
        );

        // ----------------------------------------------------------------

        // [.DOCUMENT] section
        $Conf->setSectionKeyValue(
            '.DOCUMENT', 'RESOURCE', $this->State->getDocumentStorageDSN()
        );

        // ----------------------------------------------------------------

        // [.AUTH] section
        // FIX: This is not supported yet, we take over the values of the
        // [.DOCUMENT] RESOURCE
        $Conf->setSectionKeyValue(
            '.AUTH', 'RESOURCE',
            $Conf->getSectionKeyValue('.DOCUMENT', 'RESOURCE')
        );
        $Conf->setSectionKeyValue('.AUTH', 'ENCRYPTION', 'CRYPT');
        $Conf->setSectionKeyValue(
            '.AUTH', 'ROOT_PASSWD',
            crypt($this->State->getRootUserPassword())
        );

        // ----------------------------------------------------------------

        // [.USER] section
        // FIX: This is not supported yet, we take over the values of the
        // [.DOCUMENT] RESOURCE
        $Conf->setSectionKeyValue(
            '.USER', 'RESOURCE',
            $Conf->getSectionKeyValue('.DOCUMENT', 'RESOURCE')
        );

        // ----------------------------------------------------------------

        // [RUNTIME] section
        $Conf->setSectionKeyValue(
            'RUNTIME', 'LANGUAGE_LOCALE', $this->State->getLanguageLocale()
        );
        $Conf->setSectionKeyValue(
            'RUNTIME', 'TEMPLATE_ACTIVE', $this->State->getTemplateActive()
        );

        $Conf->setSectionKeyValueBoolean(
            'RUNTIME', 'EMAIL_OBFUSCATE', $this->State->hasEmailObfuscation()
        );

        // ----------------------------------------------------------------

        // [PATH] section

        // Force the TEMP path to be the "session.save_path". See the
        // decription in the php.ini file for how a "session.save_path"
        // can look like.
        $aArr = explode(';', ini_get('session.save_path'));
        $Conf->setSectionKeyValue('PATH', 'TEMP', $aArr[sizeof($aArr)-1]);
        
        // ---

        // Store the configuration file.
        $sConfigFile = $this->State->getIncludesPath() . '/core.conf';
        $Conf->store($sConfigFile);
    }

    // --------------------------------------------------------------------

    private function storeIncludePathSettings() {
        $sIncName = $this->State->getHtdocsPath() . '/include.path';

        $sDeli = $this->State->isWindows() ? ';' : ':';

        $sContent = $this->State->getHtdocsPath();
        $sContent .= $sDeli;
        $sContent .= $this->State->getIncludesPath();
        $sContent .= $sDeli;
        $sContent .= $this->State->getMiscPath();

        try {
            $Out = new FileOutputStream($sIncName);
            $Out->write($sContent);
            $Out->close();

        } catch (IOException $ioe) {
            throw $ioe; // rethrow
        }
    }

    // --------------------------------------------------------------------

    private function sealInstallation() {

        $sSealName = $this->State->getHtdocsPath() . '/install.seal';
        $sPendName = $this->State->getHtdocsPath() . '/install.pending';

        $sContent =   '; DO NOT delete or modify this file manually!' . "\n";
        $sContent .=  'Sealed = '. date('Ymd') . "\n";
        
        try {
            $Out = new FileOutputStream($sSealName);
            $Out->write($sContent);
            $Out->close();

            @unlink($sPendName);

        } catch (IOException $ioe) {
            throw new ConfigNotWritableException(
                'Could not write configuration file '.basename($sSealName)
            );
        }
    }

} // of class

/*

    We are unemployed, happy happy unemployed
    We are unemployed, we got nothing to do, juhu
    We are unemployed, Paula, me and you

    We've got no job - job
    We've got no work - work
    We've got no job, no work and nothing else to do

    We're feelin' good - good
    We're feelin' fine - fine
    We're feelin' great and so we sing it all again

*/

?>