<?php

/**
 *
 * $Id: class.TaskMigrator.php,v 1.8 2005/05/16 22:22:43 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.8 $
 *
 */

/**
 * coWiki - Installer task class
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 */
class TaskMigrator extends AbstractTask {

    private $nMigCount = 0;

    // --------------------------------------------------------------------

    public function __construct($Tpl) {
        parent::__construct($Tpl);
        parent::checkValidUser();
        parent::loadClassLoader();
    }

    // --------------------------------------------------------------------

    public function isRequired() {

        // Get migrator classes that are scheduled
        $aMig = $this->getMigratorClasses();

        foreach ($aMig as $k => $v) {

            // Instantiate migrator class
            $Mig = new $v($this->State);

            if ($Mig->isRequired()) {

                // Get required Migrator classes count
                $this->nMigCount++;
            }
        }

        return $this->nMigCount != 0;
    }

    // --------------------------------------------------------------------

    public function validate() {

        // Get migrator classes that are scheduled
        $aMig = $this->getMigratorClasses();

        foreach ($aMig as $k => $v) {

            try {
                // Instantiate migrator class
                $Mig = new $v($this->State);

                if ($Mig->isRequired()) {

                    // Let do it its work
                    $Mig->perform();
                }

            } catch (MigratorException $me) {
                return $this->addError($me->getExceptionMessage());

            } catch (StorageException $se) {
                return $this->addError($se->getExceptionMessage());
            }
        }

        return true;
    }

    // --------------------------------------------------------------------

    public function render() {

        // Get migrator classes that are scheduled
        $aMig = $this->getMigratorClasses();

        // ----------------------------------------------------------------

        $sStr =   '<table width="100%" cellpadding="0" cellspacing="0"';
        $sStr .=    ' border="0">';
        $sStr .=    '<tr>';
        $sStr .=      '<td>';
        $sStr .=        'Your coWiki installation is not up-to-date. ';

        if ($this->nMigCount == 1) {
            $sStr .=    'Following update is pending:';
        } else {
            $sStr .=    'Following '.$this->nMigCount.' updates are pending:';
        }

        $sStr .=      '</td>';
        $sStr .=    '</tr>';

        $sStr .=    '<tr>';
        $sStr .=      '<td>';
        $sStr .=        '<img src="img/0.gif" width="1" height="8" alt=""';
        $sStr .=        ' border="0">';
        $sStr .=      '</td>';
        $sStr .=    '</tr>';

        $sStr .=    '<tr>';
        $sStr .=      '<td>';
        $sStr .=        '<div class="default-scrollarea">';

        $sStr .=          '<ul>';

        foreach ($aMig as $k => $v) {

            // Instantiate class
            $Mig = new $v($this->State);

            if ($Mig->isRequired()) {
                $sStr .=  '<li>';
#                $sStr .=      $this->getMigrationDate($k) . ': ';
                $sStr .=  $Mig->getDescription() . '<br/>';
                $sStr .=  '</li>';
            }

        }

        $sStr .=          '</ul>';

        $sStr .=        '</div>';
        $sStr .=      '</td>';
        $sStr .=    '</tr>';

        if ($this->State->isUpdateSetup()) {
            $sStr .=    '<tr>';
            $sStr .=      '<td>';
            $sStr .=        '<img src="img/0.gif" width="1" height="8"';
            $sStr .=        ' alt="" border="0">';
            $sStr .=      '</td>';
            $sStr .=    '</tr>';

            $sStr .=    '<tr>';
            $sStr .=      '<td>';
            $sStr .=        '<table cellpadding="0" cellspacing="0"
                              border="0">';
            $sStr .=          '<tr valign="top">';
            $sStr .=            '<td class="warn">';
            $sStr .=              '<b>Warning:</b>';
            $sStr .=            '</td>';
            $sStr .=            '<td>&nbsp;&nbsp;</td>';
            $sStr .=            '<td class="warn">';
            $sStr .=              'Before you proceed, make sure that you
                                   have a backup of your <i>complete</i>
                                   database (and additionally of your whole
                                   coWiki application) at least at THIS POINT,
                                   as these updates might fail under
                                   unfortunate circumstances. No warranty
                                   here, do not complain!';
            $sStr .=            '</td>';
            $sStr .=          '</tr>';
            $sStr .=        '</table>';
            $sStr .=      '</td>';
            $sStr .=    '</tr>';
        }

        $sStr .=  '</table>';

        // Define content
        $this->Tpl->assign('CONTENT', $sStr);

        // Define informational/warning/error messages
        $this->Tpl->assign('MESSAGE', $this->getMsgQueueFormatted());

        echo $this->Tpl->parse(SETUP_PATH.'/tpl/window.default.tpl');
    }

    // --------------------------------------------------------------------

    private function getMigratorClasses() {
        static $aCache = null;

        // Data already there?
        if ($aCache != null) {
            return $aCache;
        }

        $aMig = array();
        $sDir = './class/task/migrator/';

        // Read all migrator classes
        $rDir = @opendir($sDir);

        while ($sFileName = @readdir($rDir)) {

            if (substr($sFileName, 0, 15) == 'class.Migrator_') {
                if (substr($sFileName, -4) == '.php') {
                    $sKey = substr($sFileName, 15, -4);
                    $sVal = substr($sFileName, 6, -4);

                    $aMig[$sKey] = $sVal;
                }
            }
        }

        // Sort migrator files by their keys (dates)
        ksort($aMig);

        // Remember in static variable
        $aCache = $aMig;

        return $aMig;
    }

    // --------------------------------------------------------------------

    private function getMigrationDate($sDate) {
        if ($sDate == '0') {
            return '';
        }

        return strftime(
                  '%B %d, %Y',
                  strtotime(str_replace('_', '-', $sDate))
               );
    }

    // --------------------------------------------------------------------

    private function getMigrationStamp($sDate) {
        if ($sDate == '0') {
            return 0;
        }

        return strtotime(str_replace('_', '-', $sDate));
    }

} // of class

/*
    The sun rises every morning
    But why are we so sure
    I'm looking out of the window
    Empires arise and fall

    We've been so long together
    Forgotten hopes and pleasure
    But when the time is right
    We'll open our hearts
    And disclose the treasure

    So there are you
    And here am I
    Let's realize our dream
    The time is right
    Try to read between the lines

    We've been running
    Over battlefields
    From century to century
    Nothing wrong nothing right
    There was no day, there was no night

    We took every painfull step
    Life for life and side by side
    But finally look around
    Some are lost and some are found
    Lass' die Erinnerung erwachen
    I love you

    So here are you
    And here am I
    Let's realize our dream the time is right
    I'm still waiting for you
*/

?>