<?php

/**
 *
 * $Id: class.Migrator_20030910_01.php,v 1.1 2005/05/16 22:25:23 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.1 $
 *
 */

/**
 * If required, this Migrator creates a database connector user and the
 * basic database scheme.
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 */
class Migrator_20030910_01 implements MigratorInterface {

    private
        $State = null;

    // --------------------------------------------------------------------

    public function __construct($StateHolder) {
        $this->State = $StateHolder;
    }

    // --------------------------------------------------------------------

    /**
     *  Get a short description of what this Migrator is going to do
     */
    public function getDescription() {
        return 'Creation (or update) of the MySQL database connector user.';
    }

    // --------------------------------------------------------------------

    /**
     *  Check if this Migrator has to be performed, its task is:
     *
     *  Create (or update) the database MySQL connector user
     */
    public function isRequired() {

        // Get the resource DSN (database connection details)
        $UriInfo = new UriInfo($this->State->getDocumentStorageDSN());

        // Do we use MySQL? Check if this Migrator can/should generally
        // handle the requested connector scheme. If not, this Migrator is
        // "not required".
        switch ($UriInfo->get('scheme')) {
            case 'mysql':
            case 'mysql+innodb':
            case 'mysqli':
            case 'mysqli+innodb':
                break;

            default:
                // Not a known MySQL scheme, Migrator not required
                return false;
        }

        return $this->State->shallCreateNewConnectorUser();
    }

    // --------------------------------------------------------------------

    /**
     *  Perform the Migrator task
     */
    public function perform() {

        // Get the resource DSN (database connection details)
        $UriInfo = new UriInfo($this->State->getDocumentStorageDSN());

        $sUser = $UriInfo->get('user');
        $sPass = $UriInfo->get('pass');
        $sHost = $UriInfo->get('host');
        $sDB   = $UriInfo->get('basepath');

        // ---

        // Connect to database with _root_ (DBA) crendentials!
        $sResource = $this->State->getDocumentStorageRootDSN();
        $Storage = StorageFactory::getInstance()
                        ->create(new UriInfo($sResource));

        // ---

        // Let the database create an encrypted password
        $aData = $Storage->fetchArrayAndFreeResult(
                    $Storage->query(
                        "SELECT PASSWORD('".$sPass."') AS new_pass"
                    )
                 );

        // Fields for 'mysql.user'
        $aFields = array(
            'host'      =>  $sHost,
            'user'      =>  $sUser,
            'password'  =>  $aData['new_pass']
        );

        // ---

        $Storage->begin();

        try {

            // Check if connector user is already in 'mysql.user'
            $sQuery = "SELECT COUNT(password) AS rec_count
                         FROM mysql.user
                        WHERE user = '".$sUser."'";

            $aData = $Storage->fetchArrayAndFreeResult(
                        $Storage->query($sQuery)
                     );

            // No such user yet, insert him/her
            if ((int)$aData['rec_count'] == 0) {

                // Set further data for insert
                $aInsert = array(
                    'table'  => 'mysql.user',
                    'fields' => $aFields
                );
                $Storage->insert($aInsert);

            } else {

                // User already there, set further data for update
                $aUpdate = array(
                    'table'  => 'mysql.user',
                    'fields' => $aFields,
                    'where'  => "user = '".$sUser."'"
                );
                $Storage->update($aUpdate);
            }

            // ---

            // Insert user <-> database permissions
            $aRemove = array(
                'table'  => 'mysql.db',
                'where'  => "db = '".$sDB."'"
            );
            $Storage->remove($aRemove);

            // Fields for 'mysql.db'
            $aFields = array(
                'host'                  =>  $sHost,
                'user'                  =>  $sUser,
                'db'                    =>  $sDB,
                'select_priv'           =>  'Y',
                'insert_priv'           =>  'Y',
                'update_priv'           =>  'Y',
                'delete_priv'           =>  'Y',
                'create_priv'           =>  'Y',
                'drop_priv'             =>  'Y',
                'references_priv'       =>  'Y',
                'index_priv'            =>  'Y',
                'alter_priv'            =>  'Y',
                'create_tmp_table_priv' =>  'Y',
                'lock_tables_priv'      =>  'Y'
            );

            $aInsert = array(
                'table'  => 'mysql.db',
                'fields' => $aFields,
                'where'  => "db = '".$sDB."'"
            );
            $Storage->insert($aInsert);
            
        } catch (StorageException $se) {

            // End transaction
            $Storage->rollback();
            
            // Rethrow
            throw $se;
        }

        // End transaction
        $Storage->commit();

        // ------------------------------------------------------------

        // Done
        $this->State->setShallCreateNewConnectorUser(false);

        // Reload the database, initialize settings
        $Storage->reload();
    }

} // of class

?>