<?php

/**
 *
 * $Id: class.StorageFactory.php,v 1.23 2005/05/19 23:55:05 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     dao
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.23 $
 *
 */

/**
 * coWiki - Storage factory class
 *
 * @package     dao
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 *
 * @todo        [D11N]  Complete documentation
 */
class StorageFactory extends Object {

    protected static
        $Instance = null;

    // --------------------------------------------------------------------

    /**
     * Get instance
     *
     * @access  public
     * @return  object
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    public function getInstance() {
        if (!self::$Instance) {
            self::$Instance = new StorageFactory;
        }
        return self::$Instance;
    }

    // --------------------------------------------------------------------

    /**
     * Create doc storage
     *
     * @access  protected
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     */
    protected function __construct() {}

    // --------------------------------------------------------------------

    /**
     * Create doc storage
     *
     * @access  public
     * @param   string  Resource DSN
     * @return  object
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     *
     * @throws  StorageApiException
     * @throws  StorageConnectionFailureException
     * @throws  StorageDatabaseSelectionException
     * @throws  StorageException
     */
    public function createDocumentStorage($sResource) {
        $UriInfo = new UriInfo($sResource);

        // This method does not select (use) the database ...
        $Storage = $this->create($UriInfo);

        // ... we need to 'use database' here
        $Storage->useDatabase($UriInfo->get('basepath'));

        return $Storage;
    }

    // --------------------------------------------------------------------

    /**
     * Create user storage
     *
     * @access  public
     * @param   string  Resource DSN
     * @return  object
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     *
     * @throws  StorageApiException
     * @throws  StorageConnectionFailureException
     * @throws  StorageDatabaseSelectionException
     * @throws  StorageException
     */
    public function createUserStorage($sResource) {
        $UriInfo = new UriInfo($sResource);

        // This method does not select (use) the database ...
        $Storage = $this->create($UriInfo);

        // ... we need to 'use database' here
        $Storage->useDatabase($UriInfo->get('basepath'));

        return $Storage;
    }

    // --------------------------------------------------------------------

    /**
     * Create appropriate storage abstraction layer. The database might
     * be not chosen if it does not exist but the connection was successful.
     * The exception StorageDatabaseSelectionException is swallowed. You'll
     * need to call explicit useDatabase() after calling this method.
     *
     * @access  public
     * @param   object  UriInfo
     * @return  object  Storage
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     *
     * @throws  StorageApiException
     * @throws  StorageConnectionFailureException
     * @throws  StorageException
     */
    public function create($UriInfo) {

        $sScheme = $UriInfo->get('scheme');

        if ($sScheme == 'mysql' || $sScheme == 'mysql+innodb') {

            // New abstraction layer
            try {
                $Storage = new StorageMySQL();
    
                // Connect service
                $Storage->connect(
                    $UriInfo->get('host'),
                    $UriInfo->get('port'),
                    $UriInfo->get('user'),
                    $UriInfo->get('pass')
                );

                // Catch database selection exception first and swallow it.
                // This will allow to return valid $Storage back to caller,
                // even if the database could not be selected. This behaviour
                // is e.g. necessary for a client that a) can connect
                // but b) can not _use_ (maybe not yet existing) database.
                try {
                    $Storage->useDatabase($UriInfo->get('basepath'));
                } catch (StorageDatabaseSelectionException $sdse) {
                    // swallow
                }

                return $Storage;

            } catch (StorageConnectionFailureException $scfe) {
                throw $scfe; // rethrow, remember that this is catchable

            } catch (StorageException $se) {
                throw $se; // rethrow, remember that this is catchable
            }
        }

        // ---

        if ($sScheme == 'mysqli' || $sScheme == 'mysqli+innodb') {

            // New abstraction layer
            try {
                $Storage = new StorageMySQLi();
    
                // Connect service
                $Storage->connect(
                    $UriInfo->get('host'),
                    $UriInfo->get('port'),
                    $UriInfo->get('user'),
                    $UriInfo->get('pass')
                );

                // Catch database selection exception first and swallow it.
                // This will allow to return valid $Storage back to caller,
                // even if the database could not be selected. This behaviour
                // is e.g. necessary for a client that a) can connect
                // but b) can not _use_ (maybe not yet existing) database.
                try {
                    $Storage->useDatabase($UriInfo->get('basepath'));
                } catch (StorageDatabaseSelectionException $sdse) {
                    // swallow
                }

                return $Storage;

            } catch (StorageConnectionFailureException $scfe) {
                throw $scfe; // rethrow, remember that this is catchable

            } catch (StorageException $se) {
                throw $se; // rethrow, remember that this is catchable
            }
        }

        throw new StorageApiException(
            'Unknown API access scheme \''.$sScheme.'\'.'
        );
    }

} // of class

/*
    Es ist schon spt und alles, alles ist gesagt
    Alle Geschichten sind erzhlt
    Sicher werden wir uns nochmal wiedersehen
    Genau wie es gestern schon geschah
    
    Dann werden wir erzhlen bis tief in die Nacht
    Von Abenteuern und Herzen, die man brach
    Doch wir kennen uns so gut und wir wissen genau
    Nichts an alledem ist wahr ...
    
    Auf Wiederseh'n ...
*/

?>
