<?php

/**
 *
 * $Id: class.StorageMySQL.php,v 1.37 2005/05/26 23:42:54 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     dao
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.37 $
 *
 */

/**
 * This class is the MySQL (MyISAM/InnoDB) access abstraction layer.
 * It provides basic database access methods like INSERT, UPDATE or just
 * a simple query (SELECT). It is intentionally kept very simple and is
 * only a helper, but not a real persistence layer.
 *
 * @package     dao
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.3.0
 *
 * @todo        [D11N]  Complete documentation
 */
class StorageMySQL extends Object {

    protected $STORAGE_CONTAINER_NAME = 'MySQL';

    protected
        $rLink    = null,
        $rResult  = null;

    protected
        $sHost    = null,
        $sPort    = null,
        $sUser    = null,
        $sPass    = null,
        $sDb      = null;

    protected
        $nTrans   = 0;

    // --------------------------------------------------------------------

    /**
     * Class constructor.
     *
     * @access  public
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @throws  StorageApiException
     */
    public function __construct() {
        if (!function_exists('mysql_connect')) {
            throw new StorageApiException('MySQL not available.');
        }
    }

    // --------------------------------------------------------------------

    public function __destruct() {
        if ($this->rLink) {
            @mysql_close($this->rLink);
        }
    }

    // --------------------------------------------------------------------

    /**
     * Connect
     *
     * @access  public
     * @param   string
     * @param   string
     * @param   string
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     *
     * @throws  StorageConnectionFailureException
     * @throws  StorageException
     */
    public function connect($sHost, $sPort, $sUser, $sPass) {

        // Remember
        $this->sHost = $sHost;
        $this->sPort = $sPort;
        $this->sUser = $sUser;
        $this->sPass = $sPass;

        $sHostPort = $this->getHostString();

        if (!$this->rLink = @mysql_connect($sHostPort, $sUser, $sPass)) {

            // {{{ DEBUG }}}
            Logger::error('MySQL error '.mysql_errno().': '.mysql_error());
            throw new StorageConnectionFailureException(mysql_error());
        }

        // Do not auto commit
        $sQuery = 'SET AUTOCOMMIT=0';

        // {{{ DEBUG }}}
        Logger::sql($sQuery);

        $this->query($sQuery);
    }

    // --------------------------------------------------------------------

    /**
     * Use database
     *
     * @access  public
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     *
     * @throws  StorageDatabaseSelectionException
     */
    public function useDatabase($sDatabase) {
        // Remember
        $this->sDb = $sDatabase;

        if(!@mysql_select_db($sDatabase, $this->rLink)) {

            // {{{ DEBUG }}}
            Logger::error('MySQL error '.mysql_errno().': '.mysql_error());

            throw new StorageDatabaseSelectionException(mysql_error());
        }
    }

    // --------------------------------------------------------------------

    /**
     * Get current host
     *
     * @access  public
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     */
    public function getHost() {
        return $this->sHost;
    }

    // --------------------------------------------------------------------

    /**
     * Get current host port
     *
     * @access  public
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     */
    public function getHostPort() {
        return $this->sPort;
    }

    // --------------------------------------------------------------------

    /**
     * Get current host and port string
     *
     * @access  public
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     */
    public function getHostString() {
        if ($this->getHostPort() === null || $this->getHostPort() == '') {
            return $this->getHost();
        }
        
        return $this->getHost() . ':' . $this->getHostPort();
    }

    // --------------------------------------------------------------------

    /**
     * Get current connector user name
     *
     * @access  public
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     */
    public function getUserName() {
        return $this->sUser;
    }

    // --------------------------------------------------------------------

    /**
     * Get current connector user password
     *
     * @access  public
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     */
    public function getUserPassword() {
        return $this->sPass;
    }

    // --------------------------------------------------------------------

    /**
     * Get current database name
     *
     * @access  public
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     */
    public function getDatabase() {
        return $this->sDb;
    }

    // --------------------------------------------------------------------

    /**
     * Query
     *
     * @access  public
     * @param   string
     * @param   object
     * @return  resource
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$mQueryType"
     * @todo    [D11N]  Check return type
     *
     * @throws  StorageQueryException
     */
    public function query($sQuery, $mQueryType = null) {

        if (!$this->rResult = @mysql_query($sQuery, $this->rLink)) {

            // {{{ DEBUG }}}
            Logger::error('MySQL error '.mysql_errno().': '.mysql_error());

            throw new StorageQueryException(mysql_error());
        }

        return $this->rResult;
    }

    // --------------------------------------------------------------------

    /**
     * Fetch array
     *
     * @access  public
     * @param   integer
     * @return  array
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$rResource"
     * @todo    [D11N]  Check return type
     */
    public function fetchArray($rResource) {
        $mData = @mysql_fetch_assoc($rResource);
        if ($mData === false) {
            $this->freeResult($rResource);
        }
    
        return $mData;
    }

    // --------------------------------------------------------------------

    /**
     * Fetch one single result array and free the occupied memory
     * immediately.
     *
     * @access  public
     * @param   integer
     * @return  array
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$rResource"
     * @todo    [D11N]  Check return type
     */
    public function fetchArrayAndFreeResult($rResource) {
        $aData = $this->fetchArray($rResource);
        $this->freeResult($rResource);

        return $aData;
    }

    // --------------------------------------------------------------------

    /**
     * Fetch row
     *
     * @access  public
     * @param   integer
     * @return  array
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$rResource"
     * @todo    [D11N]  Check return type
     */
    public function fetchRow($rResource) {
        $mData = @mysql_fetch_row($rResource);
        if ($mData === false) {
            $this->freeResult($rResource);
        }
    
        return $mData;
    }

    // --------------------------------------------------------------------

    /**
     * Fetch one single result row and free the occupied memory
     * immediately.
     *
     * @access  public
     * @param   integer
     * @return  array
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$rResource"
     * @todo    [D11N]  Check return type
     */
    public function fetchRowAndFreeResult($rResource) {
        $aData = $this->fetchRow($rResource);
        $this->freeResult($rResource);

        return $aData;
    }

    // --------------------------------------------------------------------

    /**
     * Fetch object
     *
     * @access  public
     * @param   integer
     * @return  object
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$rResource"
     */
    public function fetchObject($rResource) {
        $aData = $this->fetchArray($rResource);

        if (!is_array($aData)) {
            return null;
        }

        $aKeys = array_keys($aData);
        $Obj = new Object();

        for ($i=0, $n=sizeof($aKeys); $i<$n; $i++) {
            $Obj->set($aKeys[$i], $aData[$aKeys[$i]]);
        }

        return $Obj;
    }

    // --------------------------------------------------------------------

    /**
     * Add limit to query
     *
     * @access  public
     * @param   string
     * @param   integer
     * @param   integer
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$nFrom"
     * @todo    [D11N]  Check the parameter type of "$nCount"
     * @todo    [D11N]  Check return type
     */
    public function addLimitToQuery($sQuery, $nFrom, $nCount) {
        return $sQuery . ' LIMIT ' . $nFrom . ',' . $nCount;
    }

    // --------------------------------------------------------------------

    /**
     * Get last insert id
     *
     * @access  public
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    public function getLastInsertId($sTableName) {
        return @mysql_insert_id($this->rLink);
    }

    // --------------------------------------------------------------------

    /**
     * Escape string
     *
     * @access  private
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    private function escapeString($sStr) {
        return addslashes($sStr);
    }

    // --------------------------------------------------------------------

    /**
     * Remove
     *
     * @access  public
     * @param   array
     * @return  ressource
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function remove($aData) {
        $sQuery = 'DELETE FROM '.$aData['table'].' WHERE '.$aData['where'];

        // {{{ DEBUG }}}
        Logger::sql($sQuery);

        return $this->query($sQuery, 'DELETE');
    }

    // --------------------------------------------------------------------

    /**
     * Insert
     *
     * @access  public
     * @param   array
     * @return  ressource
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function insert($aData) {
        $sQuery = 'INSERT INTO ' . $aData['table'];
        $sValue = '';

        $sQuery .= ' (';
        $sValue .= ' VALUES (';

        while (list($key, $value) = each($aData['fields'])) {
            $sQuery .= $key;
            $sQuery .= ", ";
            $sValue .= "'";
            $sValue .= $this->escapeString($value);
            $sValue .= "', ";
        }

        // Remove last comma and space
        $sQuery = substr($sQuery, 0, -2);
        $sValue = substr($sValue, 0, -2);

        $sQuery .= ') ';
        $sValue .= ') ';

        // {{{ DEBUG }}}
        Logger::sql($sQuery . $sValue);

        return $this->query($sQuery . $sValue, 'INSERT');
    }

    // --------------------------------------------------------------------

    /**
     * Update
     *
     * @access  public
     * @param   array
     * @return  ressource
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function update($aData) {
        $sQuery = 'UPDATE ' . $aData['table'] . ' SET ';

        while (list($key, $value) = each($aData['fields'])) {
            $sQuery .= $key;
            $sQuery .= "='";
            $sQuery .= $this->escapeString($value);
            $sQuery .= "', ";
        }

        // Remove last comma and space
        $sQuery = substr($sQuery, 0, -2);
        $sQuery .= ' WHERE ' . $aData['where'];

        // {{{ DEBUG }}}
        Logger::sql($sQuery);

        return $this->query($sQuery, 'UPDATE');
    }

    // --------------------------------------------------------------------

    /**
     * Free result
     *
     * @access  public
     * @param   ressource
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function freeResult($rResource) {
        @mysql_free_result($rResource);
    }

    // --------------------------------------------------------------------

    /**
     * Close current connection
     *
     * @access  public
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     */
    public function close() {
        if ($this->rLink) {
            @mysql_close($this->rLink);

            $this->rLink = null;
        }
    }

    // --------------------------------------------------------------------

    /**
     * Begin transaction
     *
     * @access  public
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     */
    public function begin() {
        if ($this->nTrans == 0) {

            // {{{ DEBUG }}}
            Logger::sql('BEGIN');

            $this->query('BEGIN');
        }

        $this->nTrans++;
    }

    // --------------------------------------------------------------------

    /**
     * Commit transaction
     *
     * @access  public
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     */
    public function commit() {
        $this->nTrans--;

        if ($this->nTrans < 0) {
            $this->rollback();

            throw new StorageException(
                'Tried to commit after rollback or no transaction started.'
            );
        }

        if ($this->nTrans == 0) {

            // {{{ DEBUG }}}
            Logger::sql('COMMIT');

            $this->query('COMMIT');
        }
    }

    // --------------------------------------------------------------------

    /**
     * Rollback transaction
     *
     * @access  public
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     */
    public function rollback() {
        $this->nTrans = 0;

        // {{{ DEBUG }}}
        Logger::sql('ROLLBACK');

        $this->query('ROLLBACK');
    }

    // --------------------------------------------------------------------

    /**
     * Provide an unique number - used to determine and avoid (multiple)
     * changes in a database record (this number is the actual
     * unix-timestamp plus a dash plus random eight cipher number)
     *
     * @access  public
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check return type
     */
    public function generateTan()  {
        mt_srand((double)microtime() * 1000000);
        return time() . '-' . mt_rand(10000000, 99999999);
    }

    // --------------------------------------------------------------------

    /**
     * Get date time as string
     *
     * @access  public
     * @param   integer
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$nStamp"
     * @todo    [D11N]  Check return type
     */
    public function getDateTimeAsString($nStamp) {
        return date('Y-m-d H:i:s', $nStamp);
    }

    // --------------------------------------------------------------------

    /**
     * Capacity of fields containing textdata (in bytes)
     *
     * @access  public
     * @return  integer
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    public function getTextCapacity() {
        return 65000;
    }

    // --------------------------------------------------------------------

    /**
     * Returns the version string of the underlying database
     *
     * @access  public
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     */
    public function getStorageVersion() {
        $aData = $this->fetchArray(
                    $this->query('SELECT VERSION() AS version')
                 );

        return $aData['version'];
    }

    // --------------------------------------------------------------------

    /**
     * Returns the name of the underlying database / storage container
     *
     * @access  public
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     */
    public function getStorageName() {
        return $this->STORAGE_CONTAINER_NAME;
    }

    // --------------------------------------------------------------------

    /**
     * Reload the database (flush privileges)
     *
     * @access  public
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     */
    public function reload() {
        $this->query('FLUSH PRIVILEGES');
    }

} // of class

/*
    With your feet on the air and your head on the ground
    Try this trick and spin it, yeah
    Your head will collapse if there's nothing in it
    And you'll ask yourself
    Where is my mind? Where is my mind? Where is my mind?

    Way out in the water, see it swimming

    I was swimming in the Caribbean
    Animals were hiding behind the rock
    Except for little fish
    When they told me east is west trying to talk to me, coy koi
    Where is my mind? Where is my mind? Where is my mind?

    Way out in the water, see it swimming

    With your feet on the air and your head on the ground
    Try this trick and spin it, yeah
    Your head will collapse if there's nothing in it
    And you'll ask yourself
    Where is my mind? Where is my mind? Where is my mind?

    Way out in the water, see it swimming

    With your feet on the air and your head on the ground
    Try this trick and spin it, yeah
*/

?>
